//Program: "magic.cc".  
//Written by Paul Landes (landes@jove.acs.unt.edu)  on 2/2/97

// Copyright (C) 1998 by Paul Landes. This code/program is free 
// software; you can redistribute it and/or modify it under the terms 
// of the GNU General Public License as published by the Free Software 
// Foundation; either version 2 of the License, or (at your option) 
// any later version. 

// Modifications:
// Sat Jul 11 13:30:33 CDT 1998: Modified command line options to be
// standard GNU type and added option for multiple matrix output.

#include <stdlib.h>
#include "mhelp.h"
#include "cmdop.h"

#define HELP "
usage: magicsqr d0 [ d1 ... dn ] [ -n, --noprint ]
                [ -c, --check ] [ -s, --sum ] [ -t, --html ]
       Copyright (C) 1998 By Paul Landes under the GNU public license.

       This program generates a magic square of size dimensions n X n
       and formats the data to certain specifications.
"
#define VERSION "Version 1.2 (Jul 11, 1998)"
#define PROG "magicsqr"

void printUsage(const char *msg = 0, int val = 1)
{
  if (msg) cerr << PROG << ": " << msg << endl << endl;
  else cerr << HELP << endl;
  exit(val);
}

class magOp : public ZCmdop
{
public:
  magOp(char **argv):help(HELP),version(VERSION),ZCmdop() {
    string errorString;
    print = true;
    check = false;
    html = false;
    sum = false;
    add('c', "check", TBOOL, &check);
    add('n', "noprint", TBOOL, &print);
    add('t', "html", TBOOL, &html);
    add('s', "sum", TBOOL, &sum);

    add('h', "help", TPRINTEXIT, &help);
    add('v', "version", TPRINTEXIT, &version);
    if (parse(argv, errorString) < 0) 
      printUsage( errorString.c_str() );
    if (html == true) print = false;
  }
  bool check, print, html, sum;
private:
  string help, version;
};

//This is the main function that does the work after the parsing has
//been done.  All the bools are options, app is the program name (this
//is used for internal errors).  Returns 1 on fail, 0 on success
int invoke(int n, magOp &op)
{
  int err;

  //instantiate a magic square clas
  MagicControl magical;
 
  //if error on initialization, return fail
  if ( magical.init(n) ) {
    cerr << PROG << ": error trying to initialize" << endl;
    return 1;
  }

  if (op.print == true) magical.printMat();                  //print matrix
  if (op.html == true) magical.printHtml(op.sum);            //print HTML

  //if sum is on, and we are not printing a html (which prints
  //this), print out the sum
  if ( (op.sum == true) && (op.html == false) )                  
    cout << "Magic Sum: " << magical.getsum() << endl;

  //depending on error code, print corresponding messages
  if (op.check == true) {
    err = magical.check();
    switch(err) {
    case 0:
      cout << "Rows and columns: Success" << endl 
	   << "Main diagonals: Success" << endl;
      break;

    case 1:
      cout << "Rows and columns: Fail" << endl 
	   << "Main diagonals: Fail" << endl;
      break;

    case 3:
      cout << "Rows and columns: Success" << endl 
	   << "Main diagonals: Fail" << endl;
      break;
    }
    return err;
  }

  return 0;
}

main(int argc, char **argv)
{
  //bool print = true, check = false, html = false, sum = false;
  magOp op(argv);
  string errorString;
  int n, ret;

  //if no command printed at all, return fail
  if ( argc <= 1) printUsage();

  for (int i = 1; i < argc; i++) {
    //get number to make magic square, and prompt error if < 3
    n = (int)strtod(argv[i], 0);
    if ( !n ) continue;
    else if (n < 3)
      printUsage("Cannot produce magic square of size 2 or less!");
    
    //doit, and return any error from process
    if ( (ret = invoke(n, op)) != 0 ) break;
  }
  return ret;
}
